@@ -1,4 +1,6 @@
-# Copyright 2011 OpenStack Foundation
+# vim: tabstop=4 shiftwidth=4 softtabstop=4
+
+# Copyright 2011 OpenStack LLC.
 # All Rights Reserved.
 #
 #    Licensed under the Apache License, Version 2.0 (the "License"); you may
@@ -17,22 +19,21 @@
 Various conveniences used for migration scripts
 """
 
-from oslo_log import log as logging
 import sqlalchemy.types
 
-from glance.i18n import _LI
+import glance.openstack.common.log as logging
 
 
 LOG = logging.getLogger(__name__)
 
 
 String = lambda length: sqlalchemy.types.String(
-    length=length, convert_unicode=False,
+    length=length, convert_unicode=False, assert_unicode=None,
     unicode_error=None, _warn_on_bytestring=False)
 
 
 Text = lambda: sqlalchemy.types.Text(
-    length=None, convert_unicode=False,
+    length=None, convert_unicode=False, assert_unicode=None,
     unicode_error=None, _warn_on_bytestring=False)
 
 
@@ -48,12 +49,6 @@ Integer = lambda: sqlalchemy.types.Integer()
 BigInteger = lambda: sqlalchemy.types.BigInteger()
 
 
-PickleType = lambda: sqlalchemy.types.PickleType()
-
-
-Numeric = lambda: sqlalchemy.types.Numeric()
-
-
 def from_migration_import(module_name, fromlist):
     """
     Import a migration file and return the module
@@ -61,7 +56,7 @@ def from_migration_import(module_name, fromlist):
     :param module_name: name of migration module to import from
         (ex: 001_add_images_table)
     :param fromlist: list of items to import (ex: define_images_table)
-    :returns: module object
+    :retval: module object
 
     This bit of ugliness warrants an explanation:
 
@@ -91,17 +86,17 @@ def from_migration_import(module_name, fromlist):
         # Refer to images table
     """
     module_path = 'glance.db.sqlalchemy.migrate_repo.versions.%s' % module_name
-    module = __import__(module_path, globals(), locals(), fromlist, 0)
+    module = __import__(module_path, globals(), locals(), fromlist, -1)
     return [getattr(module, item) for item in fromlist]
 
 
 def create_tables(tables):
     for table in tables:
-        LOG.info(_LI("creating table %(table)s"), {'table': table})
+        LOG.info(_("creating table %(table)s") % locals())
         table.create()
 
 
 def drop_tables(tables):
     for table in tables:
-        LOG.info(_LI("dropping table %(table)s"), {'table': table})
+        LOG.info(_("dropping table %(table)s") % locals())
         table.drop()
