@@ -1,4 +1,6 @@
-# Copyright 2011 OpenStack Foundation
+# vim: tabstop=4 shiftwidth=4 softtabstop=4
+
+# Copyright 2011 OpenStack LLC.
 # All Rights Reserved.
 #
 #    Licensed under the Apache License, Version 2.0 (the "License"); you may
@@ -20,7 +22,7 @@ that has atimes set.
 Assumptions
 ===========
 
-1. Cache data directory exists on a filesystem that updates atime on
+1. Cache data directory exists on a filesytem that updates atime on
    reads ('noatime' should NOT be set)
 
 2. Cache data directory exists on a filesystem that supports xattrs.
@@ -50,25 +52,25 @@ $image_cache_dir/
   invalid/
   queue/
 """
+
+from __future__ import absolute_import
 from contextlib import contextmanager
+import datetime
 import errno
 import os
 import stat
 import time
 
-from oslo_config import cfg
-from oslo_log import log as logging
-from oslo_utils import encodeutils
-from oslo_utils import excutils
-from oslo_utils import fileutils
 import xattr
 
 from glance.common import exception
-from glance.i18n import _, _LI
 from glance.image_cache.drivers import base
+from glance.openstack.common import cfg
+import glance.openstack.common.log as logging
 
 LOG = logging.getLogger(__name__)
 
+
 CONF = cfg.CONF
 
 
@@ -96,24 +98,24 @@ class Driver(base.Driver):
         image_cache_dir = self.base_dir
         fake_image_filepath = os.path.join(image_cache_dir, 'checkme')
         with open(fake_image_filepath, 'wb') as fake_file:
-            fake_file.write(b"XXX")
+            fake_file.write("XXX")
             fake_file.flush()
         try:
             set_xattr(fake_image_filepath, 'hits', '1')
-        except IOError as e:
+        except IOError, e:
             if e.errno == errno.EOPNOTSUPP:
-                msg = (_("The device housing the image cache directory "
-                         "%(image_cache_dir)s does not support xattr. It is"
-                         " likely you need to edit your fstab and add the "
-                         "user_xattr option to the appropriate line for the"
-                         " device housing the cache directory.") %
-                       {'image_cache_dir': image_cache_dir})
+                msg = _("The device housing the image cache directory "
+                        "%(image_cache_dir)s does not support xattr. It is "
+                        "likely you need to edit your fstab and add the "
+                        "user_xattr option to the appropriate line for the "
+                        "device housing the cache directory.") % locals()
                 LOG.error(msg)
-                raise exception.BadDriverConfiguration(driver_name="xattr",
+                raise exception.BadDriverConfiguration(driver="xattr",
                                                        reason=msg)
         else:
             # Cleanup after ourselves...
-            fileutils.delete_if_exists(fake_image_filepath)
+            if os.path.exists(fake_image_filepath):
+                os.unlink(fake_image_filepath)
 
     def get_cache_size(self):
         """
@@ -141,19 +143,24 @@ class Driver(base.Driver):
         """
         Returns a list of records about cached images.
         """
-        LOG.debug("Gathering cached image entries.")
+        LOG.debug(_("Gathering cached image entries."))
         entries = []
         for path in get_all_regular_files(self.base_dir):
             image_id = os.path.basename(path)
 
-            entry = {'image_id': image_id}
+            entry = {}
+            entry['image_id'] = image_id
+
             file_info = os.stat(path)
-            entry['last_modified'] = file_info[stat.ST_MTIME]
-            entry['last_accessed'] = file_info[stat.ST_ATIME]
+            entry['last_modified'] = iso8601_from_timestamp(
+                file_info[stat.ST_MTIME])
+            entry['last_accessed'] = iso8601_from_timestamp(
+                file_info[stat.ST_ATIME])
             entry['size'] = file_info[stat.ST_SIZE]
             entry['hits'] = self.get_hit_count(image_id)
 
             entries.append(entry)
+        entries.sort()  # Order by ID
         return entries
 
     def is_cached(self, image_id):
@@ -216,9 +223,9 @@ class Driver(base.Driver):
         """
         Removes all queued image files and any attributes about the images
         """
-        files = [f for f in get_all_regular_files(self.queue_dir)]
+        files = [f for f in self.get_cache_files(self.queue_dir)]
         for file in files:
-            fileutils.delete_if_exists(file)
+            os.unlink(file)
         return len(files)
 
     def delete_queued_image(self, image_id):
@@ -228,7 +235,8 @@ class Driver(base.Driver):
         :param image_id: Image ID
         """
         path = self.get_image_filepath(image_id, 'queue')
-        fileutils.delete_if_exists(path)
+        if os.path.exists(path):
+            os.unlink(path)
 
     def get_least_recently_accessed(self):
         """
@@ -265,37 +273,33 @@ class Driver(base.Driver):
             set_attr('hits', 0)
 
             final_path = self.get_image_filepath(image_id)
-            LOG.debug("Fetch finished, moving "
-                      "'%(incomplete_path)s' to '%(final_path)s'",
-                      dict(incomplete_path=incomplete_path,
-                           final_path=final_path))
+            LOG.debug(_("Fetch finished, moving "
+                        "'%(incomplete_path)s' to '%(final_path)s'"),
+                         dict(incomplete_path=incomplete_path,
+                              final_path=final_path))
             os.rename(incomplete_path, final_path)
 
             # Make sure that we "pop" the image from the queue...
             if self.is_queued(image_id):
-                LOG.debug("Removing image '%s' from queue after "
-                          "caching it.", image_id)
-                fileutils.delete_if_exists(
-                    self.get_image_filepath(image_id, 'queue'))
+                LOG.debug(_("Removing image '%s' from queue after "
+                            "caching it."), image_id)
+                os.unlink(self.get_image_filepath(image_id, 'queue'))
 
         def rollback(e):
-            set_attr('error', encodeutils.exception_to_unicode(e))
+            set_attr('error', "%s" % e)
 
             invalid_path = self.get_image_filepath(image_id, 'invalid')
-            LOG.debug("Fetch of cache file failed (%(e)s), rolling back by "
-                      "moving '%(incomplete_path)s' to "
-                      "'%(invalid_path)s'",
-                      {'e': encodeutils.exception_to_unicode(e),
-                       'incomplete_path': incomplete_path,
-                       'invalid_path': invalid_path})
+            LOG.debug(_("Fetch of cache file failed (%(e)s), rolling back by "
+                        "moving '%(incomplete_path)s' to "
+                        "'%(invalid_path)s'") % locals())
             os.rename(incomplete_path, invalid_path)
 
         try:
             with open(incomplete_path, 'wb') as cache_file:
                 yield cache_file
         except Exception as e:
-            with excutils.save_and_reraise_exception():
-                rollback(e)
+            rollback(e)
+            raise
         else:
             commit()
         finally:
@@ -331,23 +335,26 @@ class Driver(base.Driver):
         :param image_id: Image ID
         """
         if self.is_cached(image_id):
-            LOG.info(_LI("Not queueing image '%s'. Already cached."), image_id)
+            msg = _("Not queueing image '%s'. Already cached.") % image_id
+            LOG.warn(msg)
             return False
 
         if self.is_being_cached(image_id):
-            LOG.info(_LI("Not queueing image '%s'. Already being "
-                         "written to cache"), image_id)
+            msg = _("Not queueing image '%s'. Already being "
+                    "written to cache") % image_id
+            LOG.warn(msg)
             return False
 
         if self.is_queued(image_id):
-            LOG.info(_LI("Not queueing image '%s'. Already queued."), image_id)
+            msg = _("Not queueing image '%s'. Already queued.") % image_id
+            LOG.warn(msg)
             return False
 
         path = self.get_image_filepath(image_id, 'queue')
-        LOG.debug("Queueing image '%s'.", image_id)
+        LOG.debug(_("Queueing image '%s'."), image_id)
 
         # Touch the file to add it to the queue
-        with open(path, "w"):
+        with open(path, "w") as f:
             pass
 
         return True
@@ -365,7 +372,7 @@ class Driver(base.Driver):
             items.append((mtime, os.path.basename(path)))
 
         items.sort()
-        return [image_id for (modtime, image_id) in items]
+        return [image_id for (mtime, image_id) in items]
 
     def _reap_old_files(self, dirpath, entry_type, grace=None):
         now = time.time()
@@ -374,19 +381,18 @@ class Driver(base.Driver):
             mtime = os.path.getmtime(path)
             age = now - mtime
             if not grace:
-                LOG.debug("No grace period, reaping '%(path)s'"
-                          " immediately", {'path': path})
+                LOG.debug(_("No grace period, reaping '%(path)s'"
+                            " immediately"), locals())
                 delete_cached_file(path)
                 reaped += 1
             elif age > grace:
-                LOG.debug("Cache entry '%(path)s' exceeds grace period, "
-                          "(%(age)i s > %(grace)i s)",
-                          {'path': path, 'age': age, 'grace': grace})
+                LOG.debug(_("Cache entry '%(path)s' exceeds grace period, "
+                            "(%(age)i s > %(grace)i s)"), locals())
                 delete_cached_file(path)
                 reaped += 1
 
-        LOG.info(_LI("Reaped %(reaped)s %(entry_type)s cache entries"),
-                 {'reaped': reaped, 'entry_type': entry_type})
+        LOG.info(_("Reaped %(reaped)s %(entry_type)s cache entries"),
+                 locals())
         return reaped
 
     def reap_invalid(self, grace=None):
@@ -428,8 +434,12 @@ def get_all_regular_files(basepath):
 
 
 def delete_cached_file(path):
-    LOG.debug("Deleting image cache file '%s'", path)
-    fileutils.delete_if_exists(path)
+    if os.path.exists(path):
+        LOG.debug(_("Deleting image cache file '%s'"), path)
+        os.unlink(path)
+    else:
+        LOG.warn(_("Cached image file '%s' doesn't exist, unable to"
+                   " delete"), path)
 
 
 def _make_namespaced_xattr_key(key, namespace='user'):
@@ -475,9 +485,7 @@ def set_xattr(path, key, value):
     If xattrs aren't supported by the file-system, we skip setting the value.
     """
     namespaced_key = _make_namespaced_xattr_key(key)
-    if not isinstance(value, bytes):
-        value = str(value).encode('utf-8')
-    xattr.setxattr(path, namespaced_key, value)
+    xattr.setxattr(path, namespaced_key, str(value))
 
 
 def inc_xattr(path, key, n=1):
@@ -494,3 +502,7 @@ def inc_xattr(path, key, n=1):
     count = int(get_xattr(path, key))
     count += n
     set_xattr(path, key, str(count))
+
+
+def iso8601_from_timestamp(timestamp):
+    return datetime.datetime.utcfromtimestamp(timestamp).isoformat()
