@@ -15,17 +15,35 @@
 
 
 import json
-import logging
 import time
 
 import kombu.connection
 import kombu.entity
 
-from glance.common import cfg
 from glance.notifier import strategy
+from glance.openstack.common import cfg
+import glance.openstack.common.log as logging
 
+LOG = logging.getLogger(__name__)
 
-logger = logging.getLogger('glance.notifier.notify_kombu')
+rabbit_opts = [
+    cfg.StrOpt('rabbit_host', default='localhost'),
+    cfg.IntOpt('rabbit_port', default=5672),
+    cfg.BoolOpt('rabbit_use_ssl', default=False),
+    cfg.StrOpt('rabbit_userid', default='guest'),
+    cfg.StrOpt('rabbit_password', default='guest'),
+    cfg.StrOpt('rabbit_virtual_host', default='/'),
+    cfg.StrOpt('rabbit_notification_exchange', default='glance'),
+    cfg.StrOpt('rabbit_notification_topic',
+               default='glance_notifications'),
+    cfg.StrOpt('rabbit_max_retries', default=0),
+    cfg.StrOpt('rabbit_retry_backoff', default=2),
+    cfg.StrOpt('rabbit_retry_max_backoff', default=30),
+    cfg.BoolOpt('rabbit_durable_queues', default=False),
+]
+
+CONF = cfg.CONF
+CONF.register_opts(rabbit_opts)
 
 
 class KombuMaxRetriesReached(Exception):
@@ -35,32 +53,14 @@ class KombuMaxRetriesReached(Exception):
 class RabbitStrategy(strategy.Strategy):
     """A notifier that puts a message on a queue when called."""
 
-    opts = [
-        cfg.StrOpt('rabbit_host', default='localhost'),
-        cfg.IntOpt('rabbit_port', default=5672),
-        cfg.BoolOpt('rabbit_use_ssl', default=False),
-        cfg.StrOpt('rabbit_userid', default='guest'),
-        cfg.StrOpt('rabbit_password', default='guest'),
-        cfg.StrOpt('rabbit_virtual_host', default='/'),
-        cfg.StrOpt('rabbit_notification_exchange', default='glance'),
-        cfg.StrOpt('rabbit_notification_topic',
-                default='glance_notifications'),
-        cfg.StrOpt('rabbit_max_retries', default=0),
-        cfg.StrOpt('rabbit_retry_backoff', default=2),
-        cfg.StrOpt('rabbit_retry_max_backoff', default=30)
-        ]
-
-    def __init__(self, conf):
+    def __init__(self):
         """Initialize the rabbit notification strategy."""
-        self._conf = conf
-        self._conf.register_opts(self.opts)
-
-        self.topic = self._conf.rabbit_notification_topic
-        self.max_retries = self._conf.rabbit_max_retries
+        self.topic = CONF.rabbit_notification_topic
+        self.max_retries = CONF.rabbit_max_retries
         # NOTE(comstud): When reading the config file, these values end
         # up being strings, and we need them as ints.
-        self.retry_backoff = int(self._conf.rabbit_retry_backoff)
-        self.retry_max_backoff = int(self._conf.rabbit_retry_max_backoff)
+        self.retry_backoff = int(CONF.rabbit_retry_backoff)
+        self.retry_max_backoff = int(CONF.rabbit_retry_max_backoff)
 
         self.connection = None
         self.retry_attempts = 0
@@ -82,29 +82,30 @@ class RabbitStrategy(strategy.Strategy):
         caller.
         """
         log_info = {}
-        log_info['hostname'] = self._conf.rabbit_host
-        log_info['port'] = self._conf.rabbit_port
+        log_info['hostname'] = CONF.rabbit_host
+        log_info['port'] = CONF.rabbit_port
         if self.connection:
-            logger.info(_("Reconnecting to AMQP server on "
-                    "%(hostname)s:%(port)d") % log_info)
+            LOG.info(_("Reconnecting to AMQP server on "
+                       "%(hostname)s:%(port)d") % log_info)
             self._close()
         else:
-            logger.info(_("Connecting to AMQP server on "
-                    "%(hostname)s:%(port)d") % log_info)
+            LOG.info(_("Connecting to AMQP server on "
+                       "%(hostname)s:%(port)d") % log_info)
         self.connection = kombu.connection.BrokerConnection(
-                hostname=self._conf.rabbit_host,
-                port=self._conf.rabbit_port,
-                userid=self._conf.rabbit_userid,
-                password=self._conf.rabbit_password,
-                virtual_host=self._conf.rabbit_virtual_host,
-                ssl=self._conf.rabbit_use_ssl)
+                hostname=CONF.rabbit_host,
+                port=CONF.rabbit_port,
+                userid=CONF.rabbit_userid,
+                password=CONF.rabbit_password,
+                virtual_host=CONF.rabbit_virtual_host,
+                ssl=CONF.rabbit_use_ssl)
         self.connection_errors = self.connection.connection_errors
         self.connection.connect()
         self.channel = self.connection.channel()
         self.exchange = kombu.entity.Exchange(
                 channel=self.channel,
                 type="topic",
-                name=self._conf.rabbit_notification_exchange)
+                durable=CONF.rabbit_durable_queues,
+                name=CONF.rabbit_notification_exchange)
 
         # NOTE(jerdfelt): Normally the consumer would create the queues,
         # but we do this to ensure that messages don't get dropped if the
@@ -114,12 +115,12 @@ class RabbitStrategy(strategy.Strategy):
             queue = kombu.entity.Queue(
                     channel=self.channel,
                     exchange=self.exchange,
-                    durable=True,
+                    durable=CONF.rabbit_durable_queues,
                     name=routing_key,
                     routing_key=routing_key)
             queue.declare()
-        logger.info(_("Connected to AMQP server on "
-                "%(hostname)s:%(port)d") % log_info)
+        LOG.info(_("Connected to AMQP server on "
+                   "%(hostname)s:%(port)d") % log_info)
 
     def reconnect(self):
         """Handles reconnecting and re-establishing queues."""
@@ -143,13 +144,13 @@ class RabbitStrategy(strategy.Strategy):
             log_info = {}
             log_info['err_str'] = str(e)
             log_info['max_retries'] = self.max_retries
-            log_info['hostname'] = self._conf.rabbit_host
-            log_info['port'] = self._conf.rabbit_port
+            log_info['hostname'] = CONF.rabbit_host
+            log_info['port'] = CONF.rabbit_port
 
             if self.max_retries and self.retry_attempts >= self.max_retries:
-                logger.exception(_('Unable to connect to AMQP server on '
-                        '%(hostname)s:%(port)d after %(max_retries)d '
-                        'tries: %(err_str)s') % log_info)
+                LOG.exception(_('Unable to connect to AMQP server on '
+                                '%(hostname)s:%(port)d after %(max_retries)d '
+                                'tries: %(err_str)s') % log_info)
                 if self.connection:
                     self._close()
                 raise KombuMaxRetriesReached
@@ -159,16 +160,16 @@ class RabbitStrategy(strategy.Strategy):
                 sleep_time = min(sleep_time, self.retry_max_backoff)
 
             log_info['sleep_time'] = sleep_time
-            logger.exception(_('AMQP server on %(hostname)s:%(port)d is'
-                    ' unreachable: %(err_str)s. Trying again in '
-                    '%(sleep_time)d seconds.') % log_info)
+            LOG.exception(_('AMQP server on %(hostname)s:%(port)d is'
+                            ' unreachable: %(err_str)s. Trying again in '
+                            '%(sleep_time)d seconds.') % log_info)
             time.sleep(sleep_time)
 
     def log_failure(self, msg, priority):
         """Fallback to logging when we can't send to rabbit."""
         message = _('Notification with priority %(priority)s failed: '
                     'msg=%(msg)s')
-        logger.error(message % {'msg': msg, 'priority': priority})
+        LOG.error(message % {'msg': msg, 'priority': priority})
 
     def _send_message(self, msg, routing_key):
         """Send a message.  Caller needs to catch exceptions for retry."""
@@ -205,7 +206,7 @@ class RabbitStrategy(strategy.Strategy):
                 if 'timeout' not in str(e):
                     raise
 
-            logger.exception(_("Unable to send notification: %s") % str(e))
+            LOG.exception(_("Unable to send notification: %s") % str(e))
 
             try:
                 self.reconnect()
