@@ -19,17 +19,25 @@
 A simple filesystem-backed store
 """
 
+import errno
 import hashlib
-import logging
 import os
 import urlparse
 
 from glance.common import exception
+from glance.common import utils
+from glance.openstack.common import cfg
+import glance.openstack.common.log as logging
 import glance.store
 import glance.store.base
 import glance.store.location
 
-logger = logging.getLogger('glance.store.filesystem')
+LOG = logging.getLogger(__name__)
+
+datadir_opt = cfg.StrOpt('filesystem_store_datadir')
+
+CONF = cfg.CONF
+CONF.register_opt(datadir_opt)
 
 
 class StoreLocation(glance.store.location.StoreLocation):
@@ -54,8 +62,9 @@ class StoreLocation(glance.store.location.StoreLocation):
         self.scheme = pieces.scheme
         path = (pieces.netloc + pieces.path).strip()
         if path == '':
-            reason = _("No path specified")
-            raise exception.BadStoreUri(uri, reason)
+            reason = _("No path specified in URI: %s") % uri
+            LOG.error(reason)
+            raise exception.BadStoreUri('No path specified')
         self.path = path
 
 
@@ -93,41 +102,41 @@ class ChunkedFile(object):
 
 class Store(glance.store.base.Store):
 
-    def configure(self):
+    def get_schemes(self):
+        return ('file', 'filesystem')
+
+    def configure_add(self):
         """
         Configure the Store to use the stored configuration options
         Any store that needs special configuration should implement
         this method. If the store was not able to successfully configure
         itself, it should raise `exception.BadStoreConfiguration`
         """
-        self.datadir = self._option_get('filesystem_store_datadir')
+        self.datadir = CONF.filesystem_store_datadir
+        if self.datadir is None:
+            reason = (_("Could not find %s in configuration options.") %
+                      'filesystem_store_datadir')
+            LOG.error(reason)
+            raise exception.BadStoreConfiguration(store_name="filesystem",
+                                                  reason=reason)
 
         if not os.path.exists(self.datadir):
             msg = _("Directory to write image files does not exist "
                     "(%s). Creating.") % self.datadir
-            logger.info(msg)
+            LOG.info(msg)
             try:
                 os.makedirs(self.datadir)
             except IOError:
                 reason = _("Unable to create datadir: %s") % self.datadir
-                logger.error(reason)
+                LOG.error(reason)
                 raise exception.BadStoreConfiguration(store_name="filesystem",
                                                       reason=reason)
 
-    def _option_get(self, param):
-        result = self.options.get(param)
-        if not result:
-            reason = _("Could not find %s in configuration options.") % param
-            logger.error(reason)
-            raise exception.BadStoreConfiguration(store_name="filesystem",
-                                                  reason=reason)
-        return result
-
     def get(self, location):
         """
         Takes a `glance.store.location.Location` object that indicates
-        where to find the image file, and returns a generator for reading
-        the image file
+        where to find the image file, and returns a tuple of generator
+        (for reading the image file) and image_size
 
         :param location `glance.store.location.Location` object, supplied
                         from glance.store.location.get_location_from_uri()
@@ -139,8 +148,8 @@ class Store(glance.store.base.Store):
             raise exception.NotFound(_("Image file %s not found") % filepath)
         else:
             msg = _("Found image at %s. Returning in ChunkedFile.") % filepath
-            logger.debug(msg)
-            return ChunkedFile(filepath)
+            LOG.debug(msg)
+            return (ChunkedFile(filepath), None)
 
     def delete(self, location):
         """
@@ -151,21 +160,20 @@ class Store(glance.store.base.Store):
                   from glance.store.location.get_location_from_uri()
 
         :raises NotFound if image does not exist
-        :raises NotAuthorized if cannot delete because of permissions
+        :raises Forbidden if cannot delete because of permissions
         """
         loc = location.store_location
         fn = loc.path
         if os.path.exists(fn):
             try:
-                logger.debug(_("Deleting image at %(fn)s") % locals())
+                LOG.debug(_("Deleting image at %(fn)s") % locals())
                 os.unlink(fn)
             except OSError:
-                raise exception.NotAuthorized(_("You cannot delete file %s")
-                                                % fn)
+                raise exception.Forbidden(_("You cannot delete file %s") % fn)
         else:
             raise exception.NotFound(_("Image file %s does not exist") % fn)
 
-    def add(self, image_id, image_file):
+    def add(self, image_id, image_file, image_size):
         """
         Stores an image file with supplied identifier to the backend
         storage system and returns an `glance.store.ImageAddResult` object
@@ -173,6 +181,7 @@ class Store(glance.store.base.Store):
 
         :param image_id: The opaque image identifier
         :param image_file: The image data to write, as a file-like object
+        :param image_size: The size of the image data to write, in bytes
 
         :retval `glance.store.ImageAddResult` object
         :raises `glance.common.exception.Duplicate` if the image already
@@ -192,20 +201,34 @@ class Store(glance.store.base.Store):
 
         checksum = hashlib.md5()
         bytes_written = 0
-        with open(filepath, 'wb') as f:
-            while True:
-                buf = image_file.read(ChunkedFile.CHUNKSIZE)
-                if not buf:
-                    break
-                bytes_written += len(buf)
-                checksum.update(buf)
-                f.write(buf)
+        try:
+            with open(filepath, 'wb') as f:
+                for buf in utils.chunkreadable(image_file,
+                                              ChunkedFile.CHUNKSIZE):
+                    bytes_written += len(buf)
+                    checksum.update(buf)
+                    f.write(buf)
+        except IOError as e:
+            if e.errno != errno.EACCES:
+                self._delete_partial(filepath, image_id)
+            exceptions = {errno.EFBIG: exception.StorageFull(),
+                          errno.ENOSPC: exception.StorageFull(),
+                          errno.EACCES: exception.StorageWriteDenied()}
+            raise exceptions.get(e.errno, e)
+        except:
+            self._delete_partial(filepath, image_id)
+            raise
 
         checksum_hex = checksum.hexdigest()
 
-        logger.debug(_("Wrote %(bytes_written)d bytes to %(filepath)s with "
-                     "checksum %(checksum_hex)s") % locals())
+        LOG.debug(_("Wrote %(bytes_written)d bytes to %(filepath)s with "
+                    "checksum %(checksum_hex)s") % locals())
         return ('file://%s' % filepath, bytes_written, checksum_hex)
 
-
-glance.store.register_store(__name__, ['filesystem', 'file'])
+    @staticmethod
+    def _delete_partial(filepath, id):
+        try:
+            os.unlink(filepath)
+        except Exception as e:
+            msg = _('Unable to remove partial image data for image %s: %s')
+            LOG.error(msg % (id, e))
