@@ -17,19 +17,34 @@
 
 """Storage backend for S3 or Storage Servers that follow the S3 Protocol"""
 
-import logging
 import hashlib
 import httplib
+import re
 import tempfile
 import urlparse
 
-from glance.common import config
 from glance.common import exception
+from glance.common import utils
+from glance.openstack.common import cfg
+import glance.openstack.common.log as logging
 import glance.store
 import glance.store.base
 import glance.store.location
 
-logger = logging.getLogger('glance.store.s3')
+LOG = logging.getLogger(__name__)
+
+s3_opts = [
+    cfg.StrOpt('s3_store_host'),
+    cfg.StrOpt('s3_store_access_key', secret=True),
+    cfg.StrOpt('s3_store_secret_key', secret=True),
+    cfg.StrOpt('s3_store_bucket'),
+    cfg.StrOpt('s3_store_object_buffer_dir'),
+    cfg.BoolOpt('s3_store_create_bucket_on_put', default=False),
+    cfg.StrOpt('s3_store_bucket_url_format', default='subdomain'),
+    ]
+
+CONF = cfg.CONF
+CONF.register_opts(s3_opts)
 
 
 class StoreLocation(glance.store.location.StoreLocation):
@@ -97,7 +112,8 @@ class StoreLocation(glance.store.location.StoreLocation):
                     "s3+https://accesskey:secretkey@s3.amazonaws.com/bucket/"
                     "key-id"
                       )
-            raise exception.BadStoreUri(uri, reason)
+            LOG.error(_("Invalid store uri %(uri)s: %(reason)s") % locals())
+            raise exception.BadStoreUri(message=reason)
 
         pieces = urlparse.urlparse(uri)
         assert pieces.scheme in ('s3', 's3+http', 's3+https')
@@ -123,7 +139,8 @@ class StoreLocation(glance.store.location.StoreLocation):
                 self.secretkey = secret_key
             except IndexError:
                 reason = _("Badly formed S3 credentials %s") % creds
-                raise exception.BadStoreUri(uri, reason)
+                LOG.error(reason)
+                raise exception.BadStoreUri()
         else:
             self.accesskey = None
             path = entire_path
@@ -135,10 +152,11 @@ class StoreLocation(glance.store.location.StoreLocation):
                 self.s3serviceurl = '/'.join(path_parts).strip('/')
             else:
                 reason = _("Badly formed S3 URI. Missing s3 service URL.")
-                raise exception.BadStoreUri(uri, reason)
+                raise exception.BadStoreUri()
         except IndexError:
-            reason = _("Badly formed S3 URI")
-            raise exception.BadStoreUri(uri, reason)
+            reason = _("Badly formed S3 URI: %s") % uri
+            LOG.error(reason)
+            raise exception.BadStoreUri()
 
 
 class ChunkedFile(object):
@@ -187,7 +205,10 @@ class Store(glance.store.base.Store):
 
     EXAMPLE_URL = "s3://<ACCESS_KEY>:<SECRET_KEY>@<S3_URL>/<BUCKET>/<OBJ>"
 
-    def configure(self):
+    def get_schemes(self):
+        return ('s3', 's3+http', 's3+https')
+
+    def configure_add(self):
         """
         Configure the Store to use the stored configuration options
         Any store that needs special configuration should implement
@@ -214,12 +235,14 @@ class Store(glance.store.base.Store):
         else:  # Defaults http
             self.full_s3_host = 'http://' + self.s3_host
 
+        self.s3_store_object_buffer_dir = CONF.s3_store_object_buffer_dir
+
     def _option_get(self, param):
-        result = self.options.get(param)
+        result = getattr(CONF, param)
         if not result:
             reason = _("Could not find %(param)s in configuration "
                        "options.") % locals()
-            logger.error(reason)
+            LOG.error(reason)
             raise exception.BadStoreConfiguration(store_name="s3",
                                                   reason=reason)
         return result
@@ -234,12 +257,40 @@ class Store(glance.store.base.Store):
                         from glance.store.location.get_location_from_uri()
         :raises `glance.exception.NotFound` if image does not exist
         """
+        key = self._retrieve_key(location)
+
+        key.BufferSize = self.CHUNKSIZE
+
+        class ChunkedIndexable(glance.store.Indexable):
+            def another(self):
+                return (self.wrapped.fp.read(ChunkedFile.CHUNKSIZE)
+                        if self.wrapped.fp else None)
+
+        return (ChunkedIndexable(ChunkedFile(key), key.size), key.size)
+
+    def get_size(self, location):
+        """
+        Takes a `glance.store.location.Location` object that indicates
+        where to find the image file, and returns the image_size (or 0
+        if unavailable)
+
+        :param location `glance.store.location.Location` object, supplied
+                        from glance.store.location.get_location_from_uri()
+        """
+        try:
+            key = self._retrieve_key(location)
+            return key.size
+        except Exception:
+            return 0
+
+    def _retrieve_key(self, location):
         loc = location.store_location
         from boto.s3.connection import S3Connection
 
         s3_conn = S3Connection(loc.accesskey, loc.secretkey,
                                host=loc.s3serviceurl,
-                               is_secure=(loc.scheme == 's3+https'))
+                               is_secure=(loc.scheme == 's3+https'),
+                               calling_format=get_calling_format())
         bucket_obj = get_bucket(s3_conn, loc.bucket)
 
         key = get_key(bucket_obj, loc.key)
@@ -249,15 +300,9 @@ class Store(glance.store.base.Store):
                 "key=%(obj_name)s)") % ({'s3_host': loc.s3serviceurl,
                 'accesskey': loc.accesskey, 'bucket': loc.bucket,
                 'obj_name': loc.key})
-        logger.debug(msg)
+        LOG.debug(msg)
 
-        #if expected_size and (key.size != expected_size):
-        #   msg = "Expected %s bytes, got %s" % (expected_size, key.size)
-        #   logger.error(msg)
-        #   raise glance.store.BackendException(msg)
-
-        key.BufferSize = self.CHUNKSIZE
-        return (ChunkedFile(key), key.size)
+        return key
 
     def add(self, image_id, image_file, image_size):
         """
@@ -293,24 +338,31 @@ class Store(glance.store.base.Store):
 
         s3_conn = S3Connection(loc.accesskey, loc.secretkey,
                                host=loc.s3serviceurl,
-                               is_secure=(loc.scheme == 's3+https'))
+                               is_secure=(loc.scheme == 's3+https'),
+                               calling_format=get_calling_format())
 
-        create_bucket_if_missing(self.bucket, s3_conn, self.options)
+        create_bucket_if_missing(self.bucket, s3_conn)
 
         bucket_obj = get_bucket(s3_conn, self.bucket)
         obj_name = str(image_id)
 
+        def _sanitize(uri):
+            return re.sub('//.*:.*@',
+                          '//s3_store_secret_key:s3_store_access_key@',
+                          uri)
+
         key = bucket_obj.get_key(obj_name)
         if key and key.exists():
             raise exception.Duplicate(_("S3 already has an image at "
-                                      "location %s") % loc.get_uri())
+                                      "location %s") %
+                                      _sanitize(loc.get_uri()))
 
         msg = _("Adding image object to S3 using (s3_host=%(s3_host)s, "
                 "access_key=%(access_key)s, bucket=%(bucket)s, "
                 "key=%(obj_name)s)") % ({'s3_host': self.s3_host,
                 'access_key': self.access_key, 'bucket': self.bucket,
                 'obj_name': obj_name})
-        logger.debug(msg)
+        LOG.debug(msg)
 
         key = bucket_obj.new_key(obj_name)
 
@@ -327,28 +379,28 @@ class Store(glance.store.base.Store):
         # writing the tempfile, so we don't need to call key.compute_md5()
 
         msg = _("Writing request body file to temporary file "
-                "for %s") % loc.get_uri()
-        logger.debug(msg)
-        temp_file = tempfile.NamedTemporaryFile()
+                "for %s") % _sanitize(loc.get_uri())
+        LOG.debug(msg)
 
+        tmpdir = self.s3_store_object_buffer_dir
+        temp_file = tempfile.NamedTemporaryFile(dir=tmpdir)
         checksum = hashlib.md5()
-        chunk = image_file.read(self.CHUNKSIZE)
-        while chunk:
+        for chunk in utils.chunkreadable(image_file, self.CHUNKSIZE):
             checksum.update(chunk)
             temp_file.write(chunk)
-            chunk = image_file.read(self.CHUNKSIZE)
         temp_file.flush()
 
-        msg = _("Uploading temporary file to S3 for %s") % loc.get_uri()
-        logger.debug(msg)
+        msg = (_("Uploading temporary file to S3 for %s") %
+               _sanitize(loc.get_uri()))
+        LOG.debug(msg)
 
         # OK, now upload the data into the key
         key.set_contents_from_file(open(temp_file.name, 'r+b'), replace=False)
         size = key.size
         checksum_hex = checksum.hexdigest()
 
-        logger.debug(_("Wrote %(size)d bytes to S3 key named %(obj_name)s "
-                       "with checksum %(checksum_hex)s") % locals())
+        LOG.debug(_("Wrote %(size)d bytes to S3 key named %(obj_name)s "
+                    "with checksum %(checksum_hex)s") % locals())
 
         return (loc.get_uri(), size, checksum_hex)
 
@@ -366,7 +418,8 @@ class Store(glance.store.base.Store):
         from boto.s3.connection import S3Connection
         s3_conn = S3Connection(loc.accesskey, loc.secretkey,
                                host=loc.s3serviceurl,
-                               is_secure=(loc.scheme == 's3+https'))
+                               is_secure=(loc.scheme == 's3+https'),
+                               calling_format=get_calling_format())
         bucket_obj = get_bucket(s3_conn, loc.bucket)
 
         # Close the key when we're through.
@@ -377,7 +430,7 @@ class Store(glance.store.base.Store):
                 "key=%(obj_name)s)") % ({'s3_host': loc.s3serviceurl,
                 'accesskey': loc.accesskey, 'bucket': loc.bucket,
                 'obj_name': loc.key})
-        logger.debug(msg)
+        LOG.debug(msg)
 
         return key.delete()
 
@@ -394,32 +447,45 @@ def get_bucket(conn, bucket_id):
     bucket = conn.get_bucket(bucket_id)
     if not bucket:
         msg = _("Could not find bucket with ID %(bucket_id)s") % locals()
-        logger.error(msg)
+        LOG.error(msg)
         raise exception.NotFound(msg)
 
     return bucket
 
 
-def create_bucket_if_missing(bucket, s3_conn, options):
+def get_s3_location(s3_host):
+    from boto.s3.connection import Location
+    locations = {
+        's3.amazonaws.com': Location.DEFAULT,
+        's3-eu-west-1.amazonaws.com': Location.EU,
+        's3-us-west-1.amazonaws.com': Location.USWest,
+        's3-ap-southeast-1.amazonaws.com': Location.APSoutheast,
+        's3-ap-northeast-1.amazonaws.com': Location.APNortheast,
+    }
+    # strip off scheme and port if present
+    key = re.sub('^(https?://)?(?P<host>[^:]+)(:[0-9]+)?$',
+                 '\g<host>',
+                 s3_host)
+    return locations.get(key, Location.DEFAULT)
+
+
+def create_bucket_if_missing(bucket, s3_conn):
     """
     Creates a missing bucket in S3 if the
     ``s3_store_create_bucket_on_put`` option is set.
 
     :param bucket: Name of bucket to create
     :param s3_conn: Connection to S3
-    :param options: Option mapping
     """
     from boto.exception import S3ResponseError
     try:
         s3_conn.get_bucket(bucket)
     except S3ResponseError, e:
         if e.status == httplib.NOT_FOUND:
-            add_bucket = config.get_option(options,
-                                's3_store_create_bucket_on_put',
-                                type='bool', default=False)
-            if add_bucket:
+            if CONF.s3_store_create_bucket_on_put:
+                location = get_s3_location(CONF.s3_store_host)
                 try:
-                    s3_conn.create_bucket(bucket)
+                    s3_conn.create_bucket(bucket, location=location)
                 except S3ResponseError, e:
                     msg = ("Failed to add bucket to S3.\n"
                            "Got error from S3: %(e)s" % locals())
@@ -445,9 +511,16 @@ def get_key(bucket, obj):
     key = bucket.get_key(obj)
     if not key or not key.exists():
         msg = _("Could not find key %(obj)s in bucket %(bucket)s") % locals()
-        logger.error(msg)
+        LOG.error(msg)
         raise exception.NotFound(msg)
     return key
 
 
-glance.store.register_store(__name__, ['s3', 's3+http', 's3+https'])
+def get_calling_format(bucket_format=None):
+    import boto.s3.connection
+    if bucket_format is None:
+        bucket_format = CONF.s3_store_bucket_url_format
+    if bucket_format.lower() == 'path':
+        return boto.s3.connection.OrdinaryCallingFormat()
+    else:
+        return boto.s3.connection.SubdomainCallingFormat()
