@@ -1,4 +1,6 @@
-# Copyright 2011 OpenStack Foundation
+# vim: tabstop=4 shiftwidth=4 softtabstop=4
+
+# Copyright 2011 OpenStack, LLC
 # All Rights Reserved.
 #
 #    Licensed under the Apache License, Version 2.0 (the "License"); you may
@@ -12,19 +14,18 @@
 #    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
 #    License for the specific language governing permissions and limitations
 #    under the License.
-import hashlib
-import os
 
+import hashlib
 import httplib2
-from oslo_serialization import jsonutils
-from oslo_utils import units
-from six.moves import http_client
+import json
+import os
+import tempfile
 
 from glance.tests import functional
-from glance.tests.utils import minimal_headers
+from glance.tests.utils import execute, minimal_headers, minimal_add_command
 
-FIVE_KB = 5 * units.Ki
-FIVE_GB = 5 * units.Gi
+FIVE_KB = 5 * 1024
+FIVE_GB = 5 * 1024 * 1024 * 1024
 
 
 class TestMiscellaneous(functional.FunctionalTest):
@@ -52,19 +53,19 @@ class TestMiscellaneous(functional.FunctionalTest):
 
         # 1. POST /images with public image named Image1
         # attribute and no custom properties. Verify a 200 OK is returned
-        image_data = b"*" * FIVE_KB
+        image_data = "*" * FIVE_KB
         headers = minimal_headers('Image1')
         path = "http://%s:%d/v1/images" % ("127.0.0.1", self.api_port)
         http = httplib2.Http()
         response, content = http.request(path, 'POST', headers=headers,
                                          body=image_data)
-        self.assertEqual(http_client.CREATED, response.status)
-        data = jsonutils.loads(content)
-        self.assertEqual(hashlib.md5(image_data).hexdigest(),
-                         data['image']['checksum'])
-        self.assertEqual(FIVE_KB, data['image']['size'])
-        self.assertEqual("Image1", data['image']['name'])
-        self.assertTrue(data['image']['is_public'])
+        self.assertEqual(response.status, 201)
+        data = json.loads(content)
+        self.assertEqual(data['image']['checksum'],
+                         hashlib.md5(image_data).hexdigest())
+        self.assertEqual(data['image']['size'], FIVE_KB)
+        self.assertEqual(data['image']['name'], "Image1")
+        self.assertEqual(data['image']['is_public'], True)
 
         # 2. REMOVE the image from the filesystem
         image_path = "%s/images/%s" % (self.test_dir, data['image']['id'])
@@ -76,15 +77,15 @@ class TestMiscellaneous(functional.FunctionalTest):
                                               data['image']['id'])
         http = httplib2.Http()
         response, content = http.request(path, 'HEAD')
-        self.assertEqual(http_client.OK, response.status)
-        self.assertEqual("Image1", response['x-image-meta-name'])
+        self.assertEqual(response.status, 200)
+        self.assertEqual(response['x-image-meta-name'], "Image1")
 
         # 4. GET /images/1
-        # Verify the api throws the appropriate 404 error
+        # Verify the api throws the apropriate 404 error
         path = "http://%s:%d/v1/images/1" % ("127.0.0.1", self.api_port)
         http = httplib2.Http()
         response, content = http.request(path, 'GET')
-        self.assertEqual(http_client.NOT_FOUND, response.status)
+        self.assertEqual(response.status, 404)
 
         self.stop_servers()
 
@@ -95,28 +96,67 @@ class TestMiscellaneous(functional.FunctionalTest):
 
         We start both servers daemonized.
 
-        We then use Glance API to try adding an image that does not
+        We then use curl to try adding an image that does not
         meet validation requirements on the registry server and test
-        that the error returned from the API server is appropriate
+        that the error returned from the API server to curl is appropriate
         """
         self.cleanup()
         self.start_servers()
 
         api_port = self.api_port
-        path = 'http://127.0.0.1:%d/v1/images' % api_port
+        registry_port = self.registry_port
 
-        http = httplib2.Http()
-        response, content = http.request(path, 'GET')
-        self.assertEqual(http_client.OK, response.status)
-        self.assertEqual(b'{"images": []}', content)
+        cmd = "curl -g http://127.0.0.1:%d/v1/images" % api_port
 
-        headers = {'Content-Type': 'application/octet-stream',
-                   'X-Image-Meta-Name': 'ImageName',
-                   'X-Image-Meta-Disk-Format': 'Invalid', }
-        ignored, content = http.request(path, 'POST', headers=headers)
+        exitcode, out, err = execute(cmd)
 
-        self.assertIn(b'Invalid disk format', content,
-                      "Could not find 'Invalid disk format' "
-                      "in output: %s" % content)
+        self.assertEqual(0, exitcode)
+        self.assertEqual('{"images": []}', out.strip())
+
+        cmd = ("curl -X POST -H 'Content-Type: application/octet-stream' "
+               "-H 'X-Image-Meta-Name: ImageName' "
+               "-H 'X-Image-Meta-Disk-Format: Invalid' "
+               "http://127.0.0.1:%d/v1/images" % api_port)
+        ignored, out, err = execute(cmd)
+
+        self.assertTrue('Invalid disk format' in out,
+                        "Could not find 'Invalid disk format' "
+                        "in output: %s" % out)
+
+        self.stop_servers()
+
+    def test_api_treats_size_as_a_normal_property(self):
+        """
+        A test for LP bug #825024 -- glance client currently
+        treats size as a normal property.
+        """
+
+        self.cleanup()
+        self.start_servers()
+
+        # 1. POST /images with public image named Image1
+        # attribute and no custom properties. Verify a 200 OK is returned
+        with tempfile.NamedTemporaryFile() as image_file:
+            image_file.write("XXX")
+            image_file.flush()
+            image_file_name = image_file.name
+            suffix = 'size=12345 --silent-upload < %s' % image_file_name
+            cmd = minimal_add_command(self.api_port, 'MyImage', suffix)
+
+            exitcode, out, err = execute(cmd)
+
+        image_id = out.strip().split(':')[1].strip()
+        self.assertEqual(0, exitcode)
+        self.assertTrue('Found non-settable field size. Removing.' in out)
+        self.assertTrue('Added new image with ID: %s' % image_id in out)
+
+        # 2. Verify image added as public image
+        cmd = "bin/glance --port=%d show %s" % (self.api_port, image_id)
+
+        exitcode, out, err = execute(cmd)
+
+        self.assertEqual(0, exitcode)
+        lines = out.split("\n")[2:-1]
+        self.assertFalse("12345" in out)
 
         self.stop_servers()
