@@ -1,4 +1,6 @@
-# Copyright 2012 OpenStack Foundation
+# vim: tabstop=4 shiftwidth=4 softtabstop=4
+
+# Copyright 2012 OpenStack, LLC
 # All Rights Reserved.
 #
 #    Licensed under the Apache License, Version 2.0 (the "License"); you may
@@ -13,8 +15,6 @@
 #    License for the specific language governing permissions and limitations
 #    under the License.
 
-import http.client as http
-
 from glance.common import exception
 from glance.tests import utils as test_utils
 
@@ -26,20 +26,19 @@ class GlanceExceptionTestCase(test_utils.BaseTestCase):
             message = "default message"
 
         exc = FakeGlanceException()
-        self.assertEqual('default message', str(exc))
+        self.assertEquals(unicode(exc), 'default message')
 
     def test_specified_error_msg(self):
-        msg = exception.GlanceException('test')
-        self.assertIn('test', str(msg))
+        self.assertTrue('test' in unicode(exception.GlanceException('test')))
 
     def test_default_error_msg_with_kwargs(self):
         class FakeGlanceException(exception.GlanceException):
             message = "default message: %(code)s"
 
-        exc = FakeGlanceException(code=int(http.INTERNAL_SERVER_ERROR))
-        self.assertEqual("default message: 500", str(exc))
+        exc = FakeGlanceException(code=500)
+        self.assertEquals(unicode(exc), "default message: 500")
 
     def test_specified_error_msg_with_kwargs(self):
-        msg = exception.GlanceException('test: %(code)s',
-                                        code=int(http.INTERNAL_SERVER_ERROR))
-        self.assertIn('test: 500', str(msg))
+        self.assertTrue('test: 500' in
+                        unicode(exception.GlanceException('test: %(code)s',
+                                                          code=500)))
