@@ -1,28 +1,14 @@
-#!/usr/bin/env python
-# All Rights Reserved.
-#
-#    Licensed under the Apache License, Version 2.0 (the "License"); you may
-#    not use this file except in compliance with the License. You may obtain
-#    a copy of the License at
-#
-#         http://www.apache.org/licenses/LICENSE-2.0
-#
-#    Unless required by applicable law or agreed to in writing, software
-#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
-#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
-#    License for the specific language governing permissions and limitations
-#    under the License.
+#!/usr/bin/python
 
 import sys
 
 import keystoneclient.v2_0.client
-from oslo_config import cfg
-from oslo_log import log as logging
 
 import glance.context
-import glance.db.sqlalchemy.api as db_api
-from glance.i18n import _LC, _LE, _LI
+from glance.openstack.common import cfg
+import glance.openstack.common.log as logging
 import glance.registry.context
+import glance.db.sqlalchemy.api as db_api
 
 
 LOG = logging.getLogger(__name__)
@@ -36,7 +22,7 @@ def get_owner_map(ksclient, owner_is_tenant=True):
     else:
         entities = ksclient.users.list()
     # build mapping of (user or tenant) name to id
-    return {entity.name: entity.id for entity in entities}
+    return dict([(entity.name, entity.id) for entity in entities])
 
 
 def build_image_owner_map(owner_map, db, context):
@@ -46,23 +32,20 @@ def build_image_owner_map(owner_map, db, context):
         owner_name = image['owner']
 
         if not owner_name:
-            LOG.info(_LI('Image %s has no owner. Skipping.') % image_id)
+            LOG.info('Image %s has no owner. Skipping.' % image_id)
             continue
 
         try:
             owner_id = owner_map[owner_name]
         except KeyError:
-            msg = (_LE('Image "%(image)s" owner "%(owner)s" was not found. '
-                       'Skipping.'),
-                   {'image': image_id, 'owner': owner_name})
-            LOG.error(msg)
+            msg = 'Image %s owner %s was not found. Skipping.'
+            LOG.error(msg % (image_id, owner_name))
             continue
 
         image_owner_map[image_id] = owner_id
 
-        LOG.info(_LI('Image "%(image)s" owner "%(owner)s" -> "%(owner_id)s"'),
-                 {'image': image_id, 'owner': owner_name,
-                  'owner_id': owner_id})
+        msg = 'Image %s owner %s -> %s' % (image_id, owner_name, owner_id)
+        LOG.info(msg)
 
     return image_owner_map
 
@@ -70,7 +53,7 @@ def build_image_owner_map(owner_map, db, context):
 def update_image_owners(image_owner_map, db, context):
     for (image_id, image_owner) in image_owner_map.items():
         db.image_update(context, image_id, {'owner': image_owner})
-        LOG.info(_LI('Image %s successfully updated.'), image_id)
+        LOG.info('Image %s successfully updated.' % image_id)
 
 
 if __name__ == "__main__":
@@ -85,8 +68,7 @@ if __name__ == "__main__":
         cfg.StrOpt('keystone-admin-user',
                    help='Administrative user\'s id'),
         cfg.StrOpt('keystone-admin-password',
-                   help='Administrative user\'s password',
-                   secret=True),
+                   help='Administrative user\'s password'),
     ]
     config.register_cli_opts(extra_cli_opts)
     config(project='glance', prog='glance-registry')
@@ -101,7 +83,7 @@ if __name__ == "__main__":
     admin_password = config.keystone_admin_password
 
     if not (auth_uri and admin_tenant_name and admin_user and admin_password):
-        LOG.critical(_LC('Missing authentication arguments'))
+        LOG.critical('Missing authentication arguments')
         sys.exit(1)
 
     ks = keystoneclient.v2_0.client.Client(username=admin_user,
